import { createContext, useCallback, useContext, useEffect, useState } from 'react';
const Context = createContext({ setLoading: () => { } });
const useIsLoadingContext = () => {
    const [refs, setRefs] = useState(0);
    const loading = refs !== 0;
    const setLoading = useCallback((value) => {
        setRefs((old) => (value ? old + 1 : old - 1));
    }, []);
    return [loading, setLoading];
};
const useSyncLoading = (loading) => {
    const context = useContext(Context);
    useEffect(() => {
        if (loading) {
            context.setLoading(true);
            return () => context.setLoading(false);
        }
    }, [loading, context]);
};
const useSyncLoadingSubscriber = (subscriber) => {
    const set = useSetLoading();
    useEffect(() => {
        let loading = undefined;
        subscriber.subscribe((value) => {
            if (value && !loading)
                set(true);
            else if (!value && loading)
                set(false);
            loading = value;
        });
        return () => {
            if (loading)
                set(false);
        };
    }, [set, subscriber]);
};
const useSetLoading = () => {
    const context = useContext(Context);
    return context.setLoading;
};
export { useSyncLoading, useSetLoading, useSyncLoadingSubscriber };
const LoadingState = { Context, useIsLoadingContext };
export default LoadingState;
//# sourceMappingURL=state.js.map