import React from 'react';
import { useOutsideClick } from '../../../hooks/use-outside-click';
import { Icon } from '../icon';
import { LoadingContainer } from '../loading';
import { useModalClose } from './state';
import { cls, clsIf } from '../../../util/classes';
const Basic = ({ children, title, buttons, small, closeWhenClickOutside }) => {
    const close = useModalClose();
    const ref = useOutsideClick(!!closeWhenClickOutside, close);
    return (React.createElement("div", { ref: ref, className: cls('absolute animate-opacity bg-white left-0 mb-20 mx-auto my-0 right-0 shadow-md top-20', clsIf(!!small, 'max-w-sm', 'max-w-3xl ')) },
        (title || buttons) && (React.createElement("div", { className: 'flex h-12 items-center justify-between px-4 select-none' },
            React.createElement("div", { className: 'flex-1' }, title),
            React.createElement("div", { className: 'flex gap-4 text-2xl' },
                !!buttons && buttons,
                React.createElement(Icon, { type: 'close', className: 'cursor-pointer', onClick: close })))),
        React.createElement("div", { className: 'relative' },
            React.createElement(LoadingContainer, null, children))));
};
const Center = ({ children, title, buttons, closeWhenClickOutside }) => {
    const close = useModalClose();
    const ref = useOutsideClick(!!closeWhenClickOutside, close);
    return (React.createElement("div", { className: 'flex flex-1 items-center justify-center' },
        React.createElement("div", { ref: ref, className: 'animate-opacity bg-white max-w-3xl relative shadow-md w-full' },
            (title || buttons) && (React.createElement("div", { className: 'flex h-12 items-center justify-between px-4 select-none' },
                React.createElement("div", { className: 'flex-1' }, title),
                React.createElement("div", { className: 'flex gap-4 text-xl' },
                    !!buttons && buttons,
                    React.createElement(Icon, { type: 'close', className: 'cursor-pointer', onClick: close })))),
            React.createElement(LoadingContainer, { withoutContainer: true }, children))));
};
const Right = ({ children, title, closeWhenClickOutside }) => {
    const close = useModalClose();
    const ref = useOutsideClick(!!closeWhenClickOutside, close);
    return (React.createElement("div", { className: 'h-screen overflow-hidden relative w-full' },
        React.createElement("div", { ref: ref, className: 'absolute animate-right-left bg-white h-screen min-w-[250px] p-4 right-0' },
            title && React.createElement("h2", { className: 'mb-4 text-2xl text-center text-gray-500' }, title),
            children)));
};
const AbsoluteCenter = ({ children, title, buttons, small, closeWhenClickOutside }) => {
    const close = useModalClose();
    const ref = useOutsideClick(!!closeWhenClickOutside, close);
    return (React.createElement("div", { ref: ref, className: cls('absolute animate-opacity bg-white left-0 mb-20 mx-auto my-0 right-0 shadow-md top-20', clsIf(!!small, 'max-w-sm', 'max-w-3xl ')) },
        React.createElement("div", { className: 'relative' },
            (title || buttons) && (React.createElement("div", { className: 'absolute z-20 right-0 top-0 select-none h-12 px-4 flex justify-between items-center gap-4' },
                React.createElement("div", { className: 'flex-1' }, title),
                React.createElement("div", { className: 'flex gap-4 text-xl' },
                    !!buttons && buttons,
                    React.createElement(Icon, { type: 'close', className: 'cursor-pointer', onClick: close })))),
            React.createElement(LoadingContainer, null, children))));
};
export const ModalTemplate = { Basic, Center, AbsoluteCenter, Right };
//# sourceMappingURL=templates.js.map