import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { useSyncLoading } from '../../common/loading';
import { FormState } from '../other/formState';
import { SureExit } from './sure-exit';
import { extractObjectToFormDefaults, fixFieldName, mapFieldErrors } from '../other/data-fix';
export const changedName = '__changed';
const buildOptions = ({ current }) => {
    const change = () => {
        if (!current.handler.auto)
            return;
        if (current.sendRef)
            clearInterval(current.sendRef);
        if (current.handler.autoDelay)
            current.sendRef = setTimeout(submit, current.handler.autoDelay);
        else
            submit();
    };
    const reset = () => {
        const defKeys = Object.keys(current.def);
        const valueKeys = Object.keys(current.value);
        const clear = valueKeys.filter((e) => !defKeys.includes(e));
        clear.forEach((key) => set(key, null));
        Object.entries(current.def).forEach(([key, value]) => set(key, value, '', false));
        const val = get(changedName);
        if (val && val.length == 2 && val[0] !== false)
            set(changedName, false, '', false);
    };
    const model = () => {
        var _a, _b, _c;
        const result = {};
        Object.entries(current.value).forEach(([key, value]) => {
            //region handle changes
            if (key === changedName)
                return;
            //endregion
            let obj = result;
            let subKey = '';
            if (key.indexOf('.') !== -1) {
                const spl = key.split('.');
                for (let i = 0; i < spl.length; i++) {
                    const str = spl[i];
                    if (i + 1 === spl.length)
                        subKey = str;
                    else
                        obj = obj[str] ? obj[str] : (obj[str] = {});
                }
            }
            else {
                subKey = key;
            }
            obj[subKey] = value[0];
        });
        return (_c = (_b = (_a = current.handler).custom) === null || _b === void 0 ? void 0 : _b.call(_a, result)) !== null && _c !== void 0 ? _c : result;
    };
    const load = (entity) => {
        if (entity && typeof entity === 'object')
            current.def = extractObjectToFormDefaults(entity);
        reset();
    };
    const get = (name) => current.value[name];
    const set = (name, data, error = '', changed = true) => {
        const field = [data, error];
        current.value[name] = field;
        const callbacks = current.listening[name];
        if (callbacks)
            callbacks.forEach((e) => e(field));
        //region handle changes
        if (changed && name !== changedName) {
            const val = get(changedName);
            const has = current.changes.has(name);
            const backToDefault = data === current.def[name];
            if (has && backToDefault)
                current.changes.delete(name);
            else if (!has && !backToDefault)
                current.changes.add(name);
            const newValue = current.changes.size !== 0;
            if (!val || val.length !== 2 || newValue !== val[0])
                set(changedName, newValue, '', false);
        }
        //endregion
        change();
    };
    const error = (errors) => {
        for (const key in errors) {
            const field = current.value[key];
            set(key, field ? field[0] : null, errors[key]);
        }
    };
    const subscribe = (name, callback, def) => {
        if (!current.listening[name])
            current.listening[name] = [];
        if (!current.value[name])
            current.value[name] = [def, ''];
        const callbacks = current.listening[name];
        callback(current.value[name]);
        callbacks.push(callback);
    };
    const unsubscribe = (callback) => {
        Object.entries(current.listening).forEach(([, value]) => {
            for (let i = 0; i < value.length; i++)
                if (value[i] === callback)
                    value.splice(i, 1);
        });
    };
    const submit = (e) => {
        e === null || e === void 0 ? void 0 : e.preventDefault();
        const data = model();
        if (!current.handler.hideLoading)
            current.setLoading(true);
        const ret = current.handler.submit((data || {}));
        if (ret) {
            ret.then((model) => {
                var _a, _b;
                current.setLoading(false);
                set(changedName, false, '', false);
                const newData = (_b = (_a = current.handler).success) === null || _b === void 0 ? void 0 : _b.call(_a, model, data);
                if (newData)
                    load(newData);
            }).catch((reason) => {
                var _a, _b;
                current.setLoading(false);
                if (reason.errors) {
                    fixFieldName(reason.errors);
                    error(mapFieldErrors(reason.errors));
                    if (current.handler.fieldError)
                        current.handler.fieldError(reason.errors);
                }
                else {
                    (_b = (_a = current.handler).catch) === null || _b === void 0 ? void 0 : _b.call(_a, reason);
                }
            });
        }
        else {
            current.setLoading(false);
        }
    };
    return { reset, model, load, set, get, error, subscribe, unsubscribe, submit };
};
export const Form = ({ children, handler, className }) => {
    const [loading, setLoading] = useState(false);
    const data = useRef({
        def: {},
        value: {},
        sendRef: 0,
        setLoading,
        handler: handler,
        changes: new Set(),
        listening: {},
    });
    data.current.handler = handler;
    useSyncLoading(loading);
    const options = useMemo(() => buildOptions(data), []);
    const reset = useCallback(() => options.reset(), [options]);
    const load = handler.load;
    useEffect(() => {
        if (load) {
            if (!data.current.handler.hideLoading)
                setLoading(true);
            load()
                .then(options.load)
                .finally(() => setLoading(false));
        }
    }, [load, options]);
    return (React.createElement(FormState.Provider, { value: options },
        handler.warnOnExit && React.createElement(SureExit, { message: '\u00C9 poss\u00EDvel que as altera\u00E7\u00F5es feitas n\u00E3o sejam salvas.' }),
        React.createElement("form", { onSubmit: options.submit, onReset: reset, className: className }, children)));
};
//# sourceMappingURL=form.js.map