import { useContext, useEffect, useRef, useState, } from 'react';
import { FormContext } from '../form-context';
export const useRegisterField = (name, transformer) => {
    const context = useContext(FormContext);
    const [error, setError] = useState('');
    const ref = useRef(null);
    useEffect(() => {
        if (!ref.current)
            return;
        const current = ref.current;
        const onFieldChange = () => context.set(name, transformer ? transformer.fromString(current.value) : current.value);
        const onContextChange = (field) => {
            current.value = transformer ? transformer.toString(field[0]) : field[0];
            setError(field[1]);
        };
        context.subscribe(name, onContextChange, transformer === null || transformer === void 0 ? void 0 : transformer.fromString(''));
        current.addEventListener('input', onFieldChange);
        return () => {
            context.unsubscribe(onContextChange);
            current.removeEventListener('input', onFieldChange);
        };
    }, [ref, context, name, transformer]);
    return { error, ref };
};
//# sourceMappingURL=use-register-field.js.map