import React, { useCallback, useRef } from 'react';
import { useSubscriber } from '../../../../hooks/use-subscriber';
import { cls } from '../../../../util/classes';
import { useField } from '../../hooks/use-field';
import { InputBorder, InputError } from '../utils';
const Content = ({ filesSub, label, _error, open, disabled }) => {
    const files = useSubscriber(filesSub);
    const fast = !files ? [] : files instanceof FileList ? files : [files];
    const filled = !!files && fast.length > 0;
    return (React.createElement("div", { className: 'relative', onClick: open },
        React.createElement("div", { className: cls('border border-transparent common-floating flex justify-between p-2 w-full', cls(filled, 'filled')) },
            files &&
                !!fast.length &&
                Array.from(fast).map((value) => (React.createElement("div", { key: value.name, className: 'inline last:mr-0 pointer select-none' }, value.name))),
            "\u00A0"),
        React.createElement(InputBorder, { _error: _error, disabled: disabled, label: label })));
};
export const File = ({ name, label, multiple }) => {
    const { subscriber: { error: _error, value: filesSub }, setValue, } = useField(name, null);
    const ref = useRef(null);
    const open = useCallback(() => {
        if (ref.current)
            ref.current.click();
    }, []);
    const change = useCallback((value) => {
        if (!value)
            setValue(null);
        else if (multiple)
            setValue(value);
        else
            setValue(value.length ? value[0] : null);
    }, [multiple, setValue]);
    return (React.createElement("div", { className: 'relative w-full' },
        React.createElement("input", { ref: ref, type: 'file', className: 'hidden', multiple: multiple, onChange: (e) => change(e.target.files) }),
        React.createElement(Content, { label: label, _error: _error, filesSub: filesSub, open: open }),
        React.createElement(InputError, { _error: _error })));
};
//# sourceMappingURL=index.js.map