import React, { useCallback, useEffect, useRef, useState } from 'react';
import { fileToBase64 } from '../../../../util/file';
import { Button } from '../../../common/button';
import { useModalClose } from '../../../common/modal/other/state';
import { ModalTemplate } from '../../../common/modal/util/templates';
const extractPoints = (event, img) => {
    const rect = img.getBoundingClientRect();
    const x = event.clientX - rect.left; //x position within the element.
    const y = event.clientY - rect.top; //y position within the element.
    return { x, y };
};
const toBase64 = (img, x, y, width, height) => {
    const { naturalWidth, naturalHeight, clientWidth, clientHeight } = img;
    const fixX = naturalWidth / clientWidth;
    const fixY = naturalHeight / clientHeight;
    x *= fixX;
    y *= fixY;
    width *= fixX;
    height *= fixY;
    if (width * height <= 0)
        return null;
    const canvas = document.createElement('canvas');
    canvas.width = width;
    canvas.height = height;
    // @ts-ignore
    canvas.getContext('2d').drawImage(img, x, y, width, height, 0, 0, width, height);
    return canvas.toDataURL('image/jpeg');
};
export const ImageCropper = ({ file, onImageChoose, aspectRatio }) => {
    const close = useModalClose();
    const ref = useRef(null);
    const draw = useRef(null);
    const [data, setData] = useState(null);
    const info = useRef({
        down: false,
        point1X: 0,
        point1Y: 0,
        point2X: 0,
        point2Y: 0,
        x: 0,
        y: 0,
        width: 0,
        height: 0,
    });
    useEffect(() => {
        fileToBase64(file, setData);
    }, [file]);
    const update = useCallback(() => {
        if (!ref.current)
            return;
        if (!draw.current)
            return;
        const pos = info.current;
        const { point1X, point2X, point1Y, point2Y } = pos;
        const { width: imageWidth, height: imageHeight } = ref.current;
        const x = Math.min(point1X, point2X);
        const y = Math.min(point1Y, point2Y);
        let width = Math.max(point1X, point2X) - x;
        let height = Math.max(point1Y, point2Y) - y;
        if (aspectRatio) {
            height = width / aspectRatio;
            width = height * aspectRatio;
        }
        if (y + height > imageHeight) {
            height = imageHeight - y;
            width = height * aspectRatio;
        }
        if (x + width > imageWidth) {
            width = imageWidth - x;
            height = width / aspectRatio;
        }
        pos.x = x;
        pos.y = y;
        pos.width = width;
        pos.height = height;
        draw.current.style.top = pos.y + 'px';
        draw.current.style.left = pos.x + 'px';
        draw.current.style.width = pos.width + 'px';
        draw.current.style.height = pos.height + 'px';
    }, [aspectRatio]);
    const mouseDown = useCallback((event) => {
        if (!ref.current)
            return;
        info.current.down = true;
        const { x, y } = extractPoints(event, ref.current);
        info.current.point1X = x;
        info.current.point1Y = y;
        info.current.point2X = x;
        info.current.point2Y = y;
        update();
    }, [update]);
    const mouseUp = useCallback(() => {
        info.current.down = false;
    }, []);
    const mouseMove = useCallback((event) => {
        if (!ref.current || !info.current.down)
            return;
        const { x, y } = extractPoints(event, ref.current);
        info.current.point2X = x;
        info.current.point2Y = y;
        update();
    }, [update]);
    const ok = useCallback(() => {
        if (!ref.current)
            return;
        const base64 = toBase64(ref.current, info.current.x, info.current.y, info.current.width, info.current.height);
        if (!base64)
            return;
        onImageChoose(base64);
        close();
    }, [close, onImageChoose]);
    return (React.createElement(ModalTemplate.Basic, { title: 'Recortar imagem' },
        React.createElement("div", { className: 'flex flex-col gap-4' },
            React.createElement("div", { className: 'relative' },
                React.createElement("div", { ref: draw, style: {
                        background: 'transparent',
                        pointerEvents: 'none',
                        userSelect: 'none',
                        border: '4px dotted green',
                        position: 'absolute',
                    } }),
                data && (React.createElement("img", { src: data, alt: '', className: 'max-w-full user-drag-none', ref: ref, onMouseDown: mouseDown, onMouseUp: mouseUp, onMouseMove: mouseMove, onMouseLeave: mouseUp }))),
            React.createElement("div", { className: 'flex justify-end pb-4 pr-4 gap-4' },
                React.createElement(Button, { pallet: 'blue_out', onClick: close }, "Cancelar"),
                React.createElement(Button, { pallet: 'blue', onClick: ok }, "Ok")))));
};
//# sourceMappingURL=modal.js.map