import { useDatasource } from '../../../../hooks/use-datasource';
import { LoadingContainer, LoadingSubscriber } from '../../../common/loading';
import React, { Fragment, useCallback, useEffect, useMemo } from 'react';
import { cls, clsIf } from '../../../../util/classes';
import { Dropdown, DropdownItem, DropdownItems } from '../../../common/dropdown';
import { Icon } from '../../../common/icon';
import { useField } from '../../hooks/use-field';
import { useRequestSettable } from '../../../../hooks/use-request';
import { useCancellablePromise } from '../../../../hooks/use-cancellable-promise';
import { useSubscriber, useSubscriberCallback } from '../../../../hooks/use-subscriber';
import { InputBorder, InputError } from '../utils';
export const Select = ({ name, label, multiple, disabled, multilevel, provider }) => {
    if (multilevel && multiple)
        throw new Error('Multilevel and multiple togheter is not supported');
    const { subscriber: requestSub, setData } = useRequestSettable(provider, useMemo(() => {
        return {
            initial: {
                total: 0,
                items: [],
            },
            data: {
                ids: [],
                page: 1,
                size: 1,
                text: '',
                sortColumn: null,
            },
        };
    }, []));
    const { setValue, subscriber: fieldSub } = useField(name, multiple ? [] : null);
    useSubscriberCallback(fieldSub.value, useCallback((value) => {
        const realValue = (value === null ? [] : Array.isArray(value) ? value : [value]);
        setData({ ids: realValue, size: realValue.length }, realValue.length === 0);
    }, [setData]));
    return (React.createElement(Dropdown, { className: 'w-full', disabled: disabled },
        React.createElement(LoadingContainer, { className: 'relative w-full', type: 'background' },
            React.createElement(LoadingSubscriber, { subscriber: requestSub.loading }),
            React.createElement(Content, { setValue: setValue, valueSb: requestSub.value, label: label, _error: fieldSub.error, disabled: disabled })),
        React.createElement(InputError, { _error: fieldSub.error }),
        React.createElement(DropdownItems, null,
            React.createElement(Options, { setValue: setValue, multiple: multiple, multilevel: multilevel, provider: provider, selectedSub: requestSub.value }))));
};
const Content = ({ setValue, _error, label, valueSb, disabled }) => {
    const { items } = useSubscriber(valueSb);
    const filled = items.length > 0;
    const clear = (event) => {
        event.stopPropagation();
        setValue((value) => (Array.isArray(value) ? [] : null));
    };
    const many = items.length > 1;
    return (React.createElement("div", { className: 'border border-transparent p-2' },
        React.createElement("div", { className: cls('common-floating flex items-center justify-between', cls(filled, 'filled')) },
            React.createElement("div", { className: 'cursor-pointer flex flex-wrap gap-2 justify-items-stretch' },
                items.map((item) => (React.createElement("div", { key: item.id, className: cls('inline-block', cls(many, 'bg-gray-200 px-1'), cls(!!disabled, 'text-gray-300')) }, item.text))),
                "\u00A0"),
            filled ? (React.createElement(Icon, { type: 'clear', className: clsIf(!!disabled, 'pointer-events-none text-gray-300', 'cursor-pointer'), onClick: clear, tabIndex: 0 })) : (React.createElement(Icon, { type: 'expand_more', className: clsIf(!!disabled, 'pointer-events-none text-gray-300', 'cursor-pointer'), tabIndex: 0 }))),
        React.createElement(InputBorder, { _error: _error, disabled: disabled, label: label })));
};
const MultilevelUp = ({ selectedSub, provider, setValue, datasource }) => {
    const { items } = useSubscriber(selectedSub);
    const item = items.length == 1 ? items[0] : null;
    const promise = useCancellablePromise(provider);
    const up = useCallback(() => {
        if (!(item === null || item === void 0 ? void 0 : item.children) && (item === null || item === void 0 ? void 0 : item.parentId)) {
            promise({ ids: [item === null || item === void 0 ? void 0 : item.parentId], size: 1, page: 1 }).then((e) => {
                const temp = (e.items.length && e.items[0]);
                setValue(temp === null || temp === void 0 ? void 0 : temp.parentId);
            });
        }
        else {
            setValue(item === null || item === void 0 ? void 0 : item.parentId);
        }
    }, [item === null || item === void 0 ? void 0 : item.children, item === null || item === void 0 ? void 0 : item.parentId, promise, setValue]);
    useEffect(() => {
        datasource.set.filter({ parentId: (item === null || item === void 0 ? void 0 : item.children) ? item === null || item === void 0 ? void 0 : item.id : item === null || item === void 0 ? void 0 : item.parentId });
    }, [datasource, item]);
    return item ? (React.createElement(Fragment, null,
        React.createElement(DropdownItem, { closeOnClick: false, onClick: up }, "..."))) : null;
};
const Options = ({ setValue, multiple, multilevel, provider, selectedSub }) => {
    const datasource = useDatasource(provider, useMemo(() => ({}), []));
    const select = useCallback((item) => {
        setValue((value) => {
            const values = selectedSub.value().items;
            const isSelected = values.find((e) => e.id === item.id);
            if (multiple) {
                const temp = Array.isArray(value) ? value : [];
                return isSelected ? temp.filter((e) => e !== item.id) : [...temp, item.id];
            }
            else {
                return isSelected ? null : item.id;
            }
        });
    }, [setValue, selectedSub, multiple]);
    return (React.createElement(Fragment, null,
        React.createElement(DropdownItem, { closeOnClick: false, className: 'bg-white flex gap-2 h-10 items-center px-2 py-2' },
            React.createElement(Icon, { type: 'search', className: 'text-2xl' }),
            React.createElement("input", { type: 'text', className: 'h-10 w-full', placeholder: 'Procurar...', onChange: (e) => datasource.set.filter({ text: e.target.value }) })),
        React.createElement(LoadingContainer, { type: 'background' },
            React.createElement(LoadingSubscriber, { subscriber: datasource.subscriber.loading }),
            multilevel && (React.createElement(MultilevelUp, { provider: provider, selectedSub: selectedSub, setValue: setValue, datasource: datasource })),
            React.createElement(ItemsBlock, { datasource: datasource, select: select, multiple: multiple, multilevel: multilevel, selectedSub: selectedSub }),
            React.createElement(LoadingBlock, { datasource: datasource }),
            React.createElement(AppendBlock, { datasource: datasource }))));
};
const ItemsBlock = ({ datasource, select, multilevel, selectedSub, multiple }) => {
    const items = useSubscriber(datasource.subscriber.items);
    const { items: currentItems } = useSubscriber(selectedSub);
    return (React.createElement(Fragment, null, items.map((item) => (React.createElement(DropdownItem, { key: item.id, closeOnClick: !(multiple || (multilevel && item.children)), onClick: () => select(item), dangerouslySetInnerHTML: { __html: item.text }, className: clsIf(!!currentItems.find((e) => e.id === item.id), '& bg-gray-300 bg-opacity-70 cursor-pointer filter', '& bg-gray-50 cursor-pointer') })))));
};
const LoadingBlock = ({ datasource }) => {
    const loading = useSubscriber(datasource.subscriber.loading);
    const items = useSubscriber(datasource.subscriber.items);
    if (items.length !== 0)
        return null;
    return (React.createElement(DropdownItem, { closeOnClick: false, className: '& text-gray-600' }, loading ? 'Carregando...' : 'Sem resultados...'));
};
const AppendBlock = ({ datasource }) => {
    const pagination = useSubscriber(datasource.subscriber.pagination);
    const items = useSubscriber(datasource.subscriber.items);
    if (items.length === pagination.total)
        return null;
    return (React.createElement(DropdownItem, { closeOnClick: false, className: 'cursor-pointer', onClick: () => alert('not implemented') }, "Carregar mais..."));
};
//# sourceMappingURL=index.js.map