const cpfOrCnpj = {
    fromString: (value) => (value ? value.replace(/\D/g, '').substr(0, 14) : ''),
    toString: (value) => {
        if (!value)
            return '';
        if (value.length <= 11)
            return cpf.toString(value);
        else
            return cnpj.toString(value);
    },
    htmlType: 'text',
};
const cpf = {
    fromString: (value) => (value ? value.replace(/\D/g, '').substr(0, 11) : ''),
    toString: (value) => {
        if (!value)
            return '';
        let working = '';
        for (let i = 0; i < value.length; i++) {
            if (i === 3)
                working += '.';
            else if (i === 6)
                working += '.';
            else if (i === 9)
                working += '-';
            working += value[i];
            if (i === 10)
                break;
        }
        return working;
    },
    htmlType: 'text',
};
const text = {
    fromString: (value) => (value === undefined || value === null ? '' : value),
    toString: (value) => (value === undefined || value === null ? '' : value),
};
const number = {
    fromString: (value) => (value === undefined || value === null || value === '' ? null : +value),
    toString: (value) => (value === undefined || value === null ? '' : value.toString()),
};
const date = {
    fromString: (value) => (value === '' ? null : value),
    toString: (value) => value,
    hideEmpty: true,
};
const datetimeLocal = {
    fromString: (value) => (value === '' ? null : value),
    toString: (value) => value !== null && value !== void 0 ? value : '',
    htmlType: 'datetime-local',
    hideEmpty: true,
};
const cnpj = {
    fromString: (value) => (value ? value.replace(/\D/g, '').substr(0, 14) : ''),
    toString: (value) => {
        if (!value)
            return '';
        let working = '';
        for (let i = 0; i < value.length; i++) {
            if (i === 2)
                working += '.';
            else if (i === 5)
                working += '.';
            else if (i === 8)
                working += '/';
            else if (i === 12)
                working += '-';
            working += value[i];
            if (i === 13)
                break;
        }
        return working;
    },
    htmlType: 'text',
};
export const transformers = {
    text,
    password: text,
    email: text,
    cpf,
    cnpj,
    number,
    date,
    datetimeLocal,
    cpfOrCnpj,
};
//# sourceMappingURL=transformer.js.map