import { useDatasource } from '../../../../../hooks';
import React, { Fragment, useCallback, useEffect, useMemo } from 'react';
import { DropdownItem } from '../../../../common/dropdown';
import { Icon } from '../../../../common/icon';
import { LoadingContainer, LoadingSubscriber } from '../../../../common/loading';
import { Multilevel } from './multi-level';
import { ItemsBlock } from './items-block';
import { LoadingBlock } from './loading-block';
import { AppendBlock } from './append-block';
export const Options = ({ setValue, multiple, multilevel, provider, _selected, _filter }) => {
    const datasource = useDatasource(provider, useMemo(() => ({}), []));
    useEffect(() => {
        if (!_filter)
            return;
        return _filter.subscribe((value) => datasource.set.filter(value, { fixed: true, merge: false }));
    }, [_filter, datasource.set]);
    const select = useCallback((item) => {
        setValue((value) => {
            const values = _selected.value().items;
            const isSelected = values.find((e) => e.id === item.id);
            if (multiple) {
                const temp = Array.isArray(value) ? value : [];
                return isSelected ? temp.filter((e) => e !== item.id) : [...temp, item.id];
            }
            else {
                return isSelected ? null : item.id;
            }
        });
    }, [setValue, _selected, multiple]);
    return (React.createElement(Fragment, null,
        React.createElement(DropdownItem, { closeOnClick: false, className: 'bg-white flex gap-2 h-10 items-center px-2 py-2' },
            React.createElement(Icon, { type: 'search', className: 'text-2xl' }),
            React.createElement("input", { type: 'text', className: 'h-10 w-full', placeholder: 'Procurar...', onChange: (e) => datasource.set.filter({ text: e.target.value }) })),
        React.createElement(LoadingContainer, { type: 'background' },
            React.createElement(LoadingSubscriber, { _loading: datasource.subscriber.loading }),
            multilevel && (React.createElement(Multilevel, { provider: provider, _selected: _selected, setValue: setValue, datasource: datasource })),
            React.createElement(ItemsBlock, { datasource: datasource, select: select, multiple: multiple, multilevel: multilevel, selectedSub: _selected }),
            React.createElement(LoadingBlock, { datasource: datasource }),
            React.createElement(AppendBlock, { datasource: datasource }))));
};
//# sourceMappingURL=options.js.map