import React, { useCallback, useEffect, useMemo } from 'react';
import { useRequestSettable, useSubscriberCallback } from '../../../../../hooks';
import { useField } from '../../../hooks/use-field';
import { Dropdown, DropdownItems } from '../../../../common/dropdown';
import { LoadingContainer, LoadingSubscriber } from '../../../../common/loading';
import { InputError } from '../../_shared';
import { Options } from './options';
import { Content } from './content';
export const Select = ({ name, label, multiple, disabled, multilevel, provider, _filter }) => {
    if (multilevel && multiple)
        throw new Error('Multilevel and multiple together is not supported');
    const { subscriber: { value: _selected, loading: _loading }, setData, } = useRequestSettable(provider, useMemo(() => {
        return {
            initial: {
                total: 0,
                items: [],
            },
            data: {
                ids: [],
                page: 1,
                size: 1,
                text: '',
                sortColumn: null,
            },
        };
    }, []));
    useEffect(() => {
        if (!_filter)
            return;
        return _filter.subscribe((data) => setData(data, { merge: true }));
    }, [_filter, setData]);
    const { setValue, subscriber: { value: _value, error: _error }, } = useField(name, multiple ? [] : null);
    useSubscriberCallback(_value, useCallback((value) => {
        const realValue = (value === null ? [] : Array.isArray(value) ? value : [value]);
        setData({ ids: realValue, size: realValue.length }, { lock: realValue.length === 0, merge: true });
    }, [setData]));
    useSubscriberCallback(_selected, useCallback((selected) => {
        const value = _value.value();
        if (typeof value === 'number') {
            if (selected.total !== 1 || selected.items[0].id !== value)
                setValue(null);
        }
        else if (Array.isArray(value)) {
            const all = [...value].filter((x) => selected.items.findIndex((y) => y.id == x) !== -1);
            if (all.length !== value.length)
                setValue(all);
        }
    }, [_value, setValue]));
    return (React.createElement(Dropdown, { className: 'w-full', disabled: disabled },
        React.createElement(LoadingContainer, { className: 'relative w-full', type: 'background' },
            React.createElement(LoadingSubscriber, { _loading: _loading }),
            React.createElement(Content, { setValue: setValue, _selected: _selected, label: label, _error: _error, disabled: disabled })),
        React.createElement(InputError, { _error: _error }),
        React.createElement(DropdownItems, null,
            React.createElement(Options, { setValue: setValue, _filter: _filter, multiple: multiple, multilevel: multilevel, provider: provider, _selected: _selected }))));
};
//# sourceMappingURL=select.js.map