import { useCallback, useEffect, useMemo, useRef } from 'react';
import { createCancellable } from '../../util/self/http';
export const useCancellablePromise = (provider) => {
    const [promise, cancel] = useMemo(() => {
        const cancellable = createCancellable();
        const promise = provider({ cancelToken: cancellable.token });
        return [promise, cancellable.cancel];
    }, [provider]);
    useEffect(() => cancel, [cancel]);
    return promise;
};
export const useCreateCancellablePromise = (provider) => {
    const ref = useRef(null);
    return useCallback((cancelLast = true) => {
        //make the old value useless
        if (ref.current && cancelLast)
            ref.current();
        const cancellable = createCancellable();
        const promise = provider({ cancelToken: cancellable.token });
        ref.current = cancellable.cancel;
        return promise;
    }, [provider]);
};
//# sourceMappingURL=use-cancellable-promise.js.map