import { useEffect, useRef } from 'react';
export const useDragScroll = (ref) => {
    const scrolling = useRef(false);
    useEffect(() => {
        if (!ref.current)
            return;
        const el = ref.current;
        let down = false;
        let x = 0, y = 0, left = 0, top = 0;
        const mouseDown = (e) => {
            x = e.clientX;
            y = e.clientY;
            left = el.scrollLeft;
            top = el.scrollTop;
            down = true;
        };
        const mouseUp = () => {
            setTimeout(() => (scrolling.current = false), 0);
            down = false;
        };
        const mouseMove = (e) => {
            if (down) {
                const dx = e.clientX - x;
                const dy = e.clientY - y;
                if (Math.abs(dx) * Math.abs(dy) > 20) {
                    scrolling.current = true;
                }
                el.scrollTop = top - dy;
                el.scrollLeft = left - dx;
            }
        };
        el.addEventListener('mousedown', mouseDown);
        document.addEventListener('mouseup', mouseUp);
        document.addEventListener('mousemove', mouseMove);
        return () => {
            el.removeEventListener('mousedown', mouseDown);
            document.removeEventListener('mouseup', mouseUp);
            document.removeEventListener('mousemove', mouseMove);
        };
    }, [ref]);
    return scrolling;
};
//# sourceMappingURL=use-drag-scroll.js.map