import { useEffect, useState } from 'react';
export const useFocus = (ref) => {
    const [focus, setFocus] = useState(false);
    useEffect(() => {
        const current = ref.current;
        if (!current)
            return;
        const focusIn = () => setFocus(true);
        const focusOut = () => setFocus(false);
        current.addEventListener('focusin', focusIn);
        current.addEventListener('focusout', focusOut);
        return () => {
            current.removeEventListener('focusin', focusIn);
            current.removeEventListener('focusout', focusOut);
        };
    }, [ref]);
    return focus;
};
//# sourceMappingURL=use-focus.js.map