import { createCancellable } from '../util/http';
import { useEffect, useMemo, useRef } from 'react';
import { createObservable } from './use-subscriber';
export const useDatasource = (provider, data) => {
    const observables = useMemo(() => createObservables(data), 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    []);
    const cancelRef = useRef(createCancellable());
    const providerRef = useRef(provider);
    const intervalRef = useRef(0);
    const options = useMemo(() => {
        const cancel = cancelRef.current;
        const provider = providerRef.current({ cancelToken: cancel.token });
        const { pagination, filter: { fixed, stored }, sort, loading, items, } = observables;
        const realRefresh = () => {
            const filter = Object.assign(Object.assign(Object.assign(Object.assign({}, fixed.value()), stored.value()), pagination.value()), sort.value());
            provider(filter)
                .then((data) => {
                pagination.dispatch({ total: data.total }, true);
                items.dispatch(data.items);
            })
                .catch((e) => {
                console.log(e);
            })
                .finally(() => loading.dispatch(false));
        };
        const refresh = () => {
            if (intervalRef.current)
                clearInterval(intervalRef.current);
            else
                loading.dispatch(true);
            intervalRef.current = setTimeout(() => {
                realRefresh();
                intervalRef.current = 0;
            }, 250);
        };
        const setFilter = (value, options = { fixed: true, merge: true }) => {
            if (value.page || value.size || value.sortColumn || value.sortAsc) {
                delete value.page;
                delete value.size;
                delete value.sortColumn;
                delete value.sortAsc;
            }
            if (options.fixed)
                fixed.dispatch(value, options.merge);
            else
                stored.dispatch(value, options.merge);
            pagination.dispatch({ page: 1 }, true);
            refresh();
        };
        const setPagination = (value) => {
            pagination.dispatch(value, true);
            refresh();
        };
        const setSort = (value) => {
            sort.dispatch(value, true);
            pagination.dispatch({ page: 1 }, true);
            refresh();
        };
        return {
            set: {
                filter: setFilter,
                pagination: setPagination,
                sort: setSort,
            },
            subscriber: {
                filter: {
                    fixed: fixed.subscriber,
                    stored: stored.subscriber,
                },
                items: items.subscriber,
                sort: sort.subscriber,
                pagination: pagination.subscriber,
                loading: loading.subscriber,
            },
            refresh,
        };
    }, [observables]);
    useEffect(() => {
        if (providerRef.current !== provider) {
            providerRef.current = provider;
            options.refresh();
        }
        const current = cancelRef.current;
        const { loading } = observables;
        return () => {
            current.cancel();
            loading.dispatch(false);
        };
    }, [provider, options, observables]);
    useEffect(() => {
        if (data === null || data === void 0 ? void 0 : data.fixed) {
            observables.filter.fixed.dispatch(data === null || data === void 0 ? void 0 : data.fixed, true);
            options.refresh();
        }
    }, [data === null || data === void 0 ? void 0 : data.fixed, observables, options]);
    useEffect(() => {
        const { filter: { stored }, sort, pagination, } = observables;
        options.refresh();
        return () => {
            if (!(data === null || data === void 0 ? void 0 : data.storedKey))
                return;
            const key = data.storedKey;
            setSessionStored(key ? key + '-filter' : undefined, stored.value());
            setSessionStored(key ? key + '-sort' : undefined, sort.value());
            setSessionStored(key ? key + '-pagination' : undefined, pagination.value());
        };
    }, [data === null || data === void 0 ? void 0 : data.storedKey, observables, options]);
    return options;
};
//region Private Methods
const createObservables = (data) => {
    var _a;
    const { fixed, storedKey, stored } = data !== null && data !== void 0 ? data : {};
    const filter = storedKey ? getSessionStored(storedKey + '-filter') : {};
    const sort = storedKey ? getSessionStored(storedKey + '-sort') : {};
    const pagination = storedKey ? getSessionStored(storedKey + '-pagination') : {};
    return {
        filter: {
            fixed: createObservable(Object.assign({}, fixed)),
            stored: createObservable(Object.assign(Object.assign({}, filter), stored)),
        },
        loading: createObservable(false),
        items: createObservable([]),
        sort: createObservable(Object.assign({}, sort)),
        pagination: createObservable(Object.assign({ page: 1, total: 0, size: (_a = data === null || data === void 0 ? void 0 : data.size) !== null && _a !== void 0 ? _a : 10 }, pagination)),
    };
};
const getSessionStored = (key) => {
    if (!key)
        return {};
    const json = sessionStorage.getItem(key);
    if (!json)
        return {};
    try {
        const result = JSON.parse(json);
        if (typeof result !== 'object')
            return {};
        return result;
    }
    catch (exception) {
        return {};
    }
};
const setSessionStored = (key, data) => {
    if (!key || !data)
        return;
    sessionStorage.setItem(key, JSON.stringify(data));
};
//endregion
//# sourceMappingURL=use-datasource.js.map