import { useCallback, useEffect, useMemo, useRef } from 'react';
import { useCreateCancellablePromise } from './use-cancellable-promise';
import { createObservable } from './use-subscriber';
const useRequest = (provider, request) => {
    const [_loading, _value] = useMemo(() => [createObservable(false), createObservable(request.initial)], 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    []);
    const createPromise = useCreateCancellablePromise(provider);
    const refresh = useCallback(() => {
        if (request.lock) {
            _value.dispatch(request.initial);
            return;
        }
        const promise = createPromise();
        _loading.dispatch(true);
        const data = request.data;
        promise(data)
            .then(_value.dispatch)
            .catch((error) => { var _a; return (_a = request === null || request === void 0 ? void 0 : request.catch) === null || _a === void 0 ? void 0 : _a.call(request, error); })
            .finally(() => _loading.dispatch(false));
    }, [request, createPromise, _loading, _value]);
    useEffect(refresh, [refresh]);
    return {
        refresh,
        subscriber: { loading: _loading.subscriber, value: _value.subscriber },
    };
};
const useRequestSettable = (provider, request) => {
    const ref = useRef({ data: undefined, lock: true });
    const [_loading, _value] = useMemo(() => [createObservable(false), createObservable(request.initial)], 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    []);
    const createPromise = useCreateCancellablePromise(provider);
    const refresh = useCallback(() => {
        if (ref.current.lock) {
            _value.dispatch(request.initial);
            return;
        }
        const promise = createPromise();
        _loading.dispatch(true);
        promise(Object.assign(Object.assign({}, request.data), ref.current.data))
            .then(_value.dispatch)
            .catch((error) => { var _a; return (_a = request === null || request === void 0 ? void 0 : request.catch) === null || _a === void 0 ? void 0 : _a.call(request, error); })
            .finally(() => _loading.dispatch(false));
    }, [createPromise, _loading, _value, request]);
    const setData = useCallback((value, lock) => {
        ref.current.data = value;
        ref.current.lock = lock;
        refresh();
    }, [refresh]);
    useEffect(refresh, [refresh]);
    return {
        refresh,
        setData,
        subscriber: { loading: _loading.subscriber, value: _value.subscriber },
    };
};
export { useRequest, useRequestSettable };
//# sourceMappingURL=use-request.js.map