import { useEffect, useMemo, useState } from 'react';
export const useSubscriber = (subscriber) => {
    const [value, setValue] = useState(subscriber.value());
    useEffect(() => subscriber.subscribe(setValue), [subscriber]);
    return value;
};
export const useSubscriberSelectorAsSubscriber = (subscriber, selector) => {
    const observable = useMemo(() => createObservable(selector(subscriber.value())), 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    []);
    useEffect(() => subscriber.subscribe((value) => observable.dispatch(selector(value))), [observable, selector, subscriber]);
    return observable.subscriber;
};
export const useSubscriberSelector = (subscriber, selector) => {
    const [value, setValue] = useState(selector(subscriber.value()));
    useEffect(() => subscriber.subscribe((value) => setValue(selector(value))), [selector, subscriber]);
    return value;
};
export const useSubscriberCallback = (subscriber, callback) => {
    useEffect(() => subscriber.subscribe(callback), [callback, subscriber]);
};
export const createObservable = (initial) => {
    const listening = [];
    let current = initial;
    const value = () => current;
    const dispatch = (value, merge = false) => {
        const old = current;
        current = (merge ? Object.assign(Object.assign({}, current), value) : value);
        if (current === old)
            return;
        listening.forEach((x) => x(current));
    };
    const subscriber = {
        value,
        subscribe: (callback) => {
            listening.push(callback);
            callback(current);
            return () => listening.splice(listening.indexOf(callback), 1);
        },
    };
    return { value, dispatch, subscriber };
};
//# sourceMappingURL=use-subscriber.js.map