import { useEffect } from 'react';
import axios from 'axios';
const compileParameters = (parameters) => {
    if (!parameters)
        return '';
    const compiled = Object.entries(parameters)
        .map(([key, value]) => `${key}=${value}`)
        .join('&');
    return compiled ? '?' + compiled : '';
};
const errorHandler = [];
const useHttpErrorHandler = (callback) => {
    useEffect(() => {
        errorHandler.push(callback);
        return () => {
            errorHandler.splice(errorHandler.indexOf(callback), 1);
        };
    }, [callback]);
};
const headers = {};
const useSetHeader = (key, value) => {
    useEffect(() => {
        headers[key] = value;
        return () => {
            delete headers[key];
        };
    }, [key, value]);
};
const buildFormData = (formData, data, parentKey) => {
    if (data && typeof data === 'object' && !(data instanceof Date) && !(data instanceof File)) {
        Object.keys(data).forEach((key) => {
            buildFormData(formData, data[key], parentKey ? `${parentKey}[${key}]` : key);
        });
    }
    else if (parentKey) {
        const value = data == null ? '' : data;
        formData.append(parentKey, value);
    }
};
const request = (url, method, data, info, dataType) => {
    if (method === 'GET' && data) {
        url = url + compileParameters(data);
        data = undefined;
    }
    const requestHeaders = Object.assign({}, headers);
    if (dataType === 'FORMDATA') {
        requestHeaders['Content-Type'] = 'multipart/form-data';
        const formData = new FormData();
        buildFormData(formData, data);
        data = formData;
    }
    const promise = axios
        .request(Object.assign({ method, url: url, data: data, headers: requestHeaders }, info))
        .then((x) => (!x || (x && x.data.fail) ? Promise.reject(x === null || x === void 0 ? void 0 : x.data) : x.data.data))
        .catch((error) => {
        var _a;
        if (axios.isCancel(error))
            return Promise.reject('cancelled');
        if ((_a = error.response) === null || _a === void 0 ? void 0 : _a.data) {
            errorHandler.forEach((e) => e(error.response.data));
            return Promise.reject(error.response.data);
        }
        errorHandler.forEach((e) => e(error));
        throw error;
    });
    return promise;
};
const createCancellable = () => axios.CancelToken.source();
//endregion
//region Methods
const builder = (url, method, type) => (info) => (data) => request(url, method, data, info, type);
const post = (url, type) => builder(url, 'POST', type);
const get = (url, type) => builder(url, 'GET', type);
const _delete = (url, type) => builder(url, 'DELETE', type);
const put = (url, type) => builder(url, 'PUT', type);
// const postFormData = <E>(url: string, data: any = null): Promise<E> => {
//     throw new Error('not implemented');
// };
const http = { request: request, post, get, delete: _delete, put };
export { http, useHttpErrorHandler, useSetHeader, createCancellable };
//# sourceMappingURL=http.js.map